<?php
namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Pla01900ParametroDePlanejamentoController extends \Teknisa\GER\Controllers\TeknisaCrud {
	protected $parameterBagColumns;
	protected $crudService;
	protected $dataSourceManager;
	protected $environment;
	protected $parameterBag;
	protected $dataSourceName = "pla01900_cardapio";
	protected $dataSourceToSaveCard = "pla01900_cardapio_to_save";
	protected $dataSourceToSavePers = "pla01900_personalizacao";
	protected $dataSourceToSaveCardDia = "pla01900_findMensCardDiario";
	protected $dataSourceToSaveCardSem = "pla01900_findMensCardSemanal";
	protected $dataSourceToSaveCardMen = "pla01900_findMensCardMensal";
	protected $dataSourceToSaveReqRetPrato = "pla01900_reqRetPrato";
	protected $dataSourceToSaveConfigPra = "pla01900_configPra";
	protected $dataSourceToSaveProdDist = "pla01900_producaoDistribuicao";
	protected $dataSourceToSavePla = "pla01900_planejamento";
	protected $dataSourceToSaveSafraCard = "pla01900_safraCardapio";
	protected $dataSourceToSaveElabCard = "pla01900_elaboraCardapio";
	protected $dataSourceToSavePlaParamFilial = "pla01900_planejamentoFilial";
	protected $dataSourceToSaveEfetivos = "pla01900_efetivos";
	protected $PlaParamService;
	protected $novoCodigo;
	protected $util;
	
	public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $entityManager, $PlaParamService, $novoCodigo, $util){
		$this->crudService 	                            = $crudService;
	    $this->parameterBag                             = $parameterBag;
		$this->dataSourceManager                        = $dataSourceManager;
		$this->environment 			                    = $environment;
		$this->entityManager                            = $entityManager;
		$this->Pla01900ParametroDePlanejamentoService   = $PlaParamService;
		$this->novoCodigo         		                = $novoCodigo;
		$this->util              		                = $util;

		parent::__construct($crudService, $parameterBag);
	}
	
	private function updateFiliaisFilhasPlanejamento(){
	    $connection = $this->entityManager->getConnection();
        
	    $procedure = new StoredProcedure($connection, 'ATUALIZA_PARAMPLANEJA_PDR');
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, null, Param::PARAM_TYPE_NULL));
        $procedure->execute();
	}
	
	private function updateFiliaisFilhasFilial(){
	    $connection = $this->entityManager->getConnection();
        
	    $procedure = new StoredProcedure($connection, 'ATUALIZA_PARAMFILIAL_PDR');
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, null, Param::PARAM_TYPE_NULL));
        $procedure->execute();
	}
	
    public function verifyFilial(DTO\Request\Row $request, DTO\Response $response){
        try{
            $sql = "SELECT 
                        CASE WHEN F.CDFILIAL = NVL(P.CDFILIPADRAO,F.CDFILIAL) 
                            THEN 'N' 
                            ELSE NVL(PR.IDTABPARAMFILI,'N')
                            END AS BLOQUEIA_EDICAO,
                        CASE WHEN F.CDFILIAL = P.CDFILIPADRAO 
                            THEN 'S' 
                            ELSE 'N' 
                            END AS FILIAL_PADRAO,
                        NMTABPARAMFILI
                    FROM FILIAL F 
                    LEFT JOIN PARAMFILIPADR PR ON PR.CDFILIAL = F.CDFILIAL
                        AND (PR.NMTABPARAMFILI = 'PARAMPLANEJA' OR PR.NMTABPARAMFILI = 'PARAMFILIAL')
                    CROSS JOIN PARAMGERAL P 
                    WHERE F.CDFILIAL = :P_CDFILIAL
                    ORDER BY NMTABPARAMFILI";
            
            $row = $request->getRow(); 
            
            $params = array(
                "P_CDFILIAL" => $row["CDFILIAL"]
            );
            
            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            $result = $query->fetchAll();
            $response->addDataSet(new DataSet("EDICAO", $result));
            
            return $result;
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
	public function ValidEditMask(DTO\Request\Filter $request, DTO\Response $response) {
		$sql = "SELECT CDPRATO FROM PRATO WHERE ROWNUM = 1";
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute();
        $result = $query->fetchAll();
        $response->addDataSet(new DataSet("VALIDMASK", $result));
	}
	
	public function saveCardapio(DTO\Request\DataSet $request, DTO\Response $response) {
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceToSaveCard);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceToSaveCard, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
        
    }
    
    public function savePersonalizacao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceToSavePers);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceToSavePers, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
        
    }
    
    public function ValidRangeSafra(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
		$sql = "SELECT VRLIMINFSAFR, VRLIMSUPSAFR, NRCORSINSAFR, NMSINASAFRA FROM SINASAFRA WHERE ((:P_VRLIMINFSAFR >= VRLIMINFSAFR) AND (:P_VRLIMINFSAFR < VRLIMSUPSAFR)) OR ((:P_VRLIMSUPSAFR > VRLIMINFSAFR) AND (:P_VRLIMSUPSAFR <= VRLIMSUPSAFR)) ORDER BY VRLIMINFSAFR";
		$params = array("P_VRLIMINFSAFR" => $row['VRLIMINFSAFR'],
		                "P_VRLIMSUPSAFR" => $row['VRLIMSUPSAFR']);
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        $response->addDataSet(new DataSet("VALIDRANGE", $result));
	}
	
	private function getAutPerfilAcesso(){
	    $dataSourceName = 'autperfilacesso';
	    $result = $this->crudService->getFilteredDataSet($dataSourceName, array())->getRows();
        return $result;
	}
	
	private function insertAutPerfilAcesso($connection, $row){
	    $sql = "INSERT INTO AUTPERFILACESSO (
	                NRAUTPERFACESSO, 
	                NRPERFILACESSO, 
	                IDAUTPERFACESSO, 
	                DSAUTPERFACESSO, 
	                NRORG, 
	                DTINCLUSAO, 
	                DTULTATU, 
	                NRORGINCLUSAO, 
	                CDOPERINCLUSAO, 
	                NRORGULTATU, 
	                CDOPERULTATU
	           ) VALUES (
	                :NRAUTPERFACESSO,
                    :NRPERFILACESSO,
                    :IDAUTPERFACESSO,
                    :DSAUTPERFACESSO,
                    :NRORG,
                    SYSDATE,
                    SYSDATE,
                    :NRORGINCLUSAO,
                    :CDOPERINCLUSAO,
                    :NRORGULTATU,
                    :CDOPERULTATU
	           )";
	   $params = array(
            "NRAUTPERFACESSO" => $row['NRAUTPERFACESSO'],
            "NRPERFILACESSO"  => $row['NRPERFILACESSO'],
            "IDAUTPERFACESSO" => $row['IDAUTPERFACESSO'],
            "DSAUTPERFACESSO" => $row['DSAUTPERFACESSO'],
            "NRORG"           => $row['NRORG'],
            "NRORGINCLUSAO"   => $row['NRORGINCLUSAO'],
            "CDOPERINCLUSAO"  => $row['CDOPERINCLUSAO'],
            "NRORGULTATU"     => $row['NRORGULTATU'],
            "CDOPERULTATU"    => $row['CDOPERULTATU']
	   );
	   
	    $stmt = $connection->prepare($sql);
        $stmt->execute($params);
	}
	
	public function deleteAutPerfilAcesso($connection, $row){
	    $sql = "DELETE FROM AUTPERFILACESSO WHERE IDAUTPERFACESSO = :IDAUTPERFACESSO AND NRORG = :NRORG";
	    $params = array(
	        "IDAUTPERFACESSO" => $row['IDAUTPERFACESSO'],
	        "NRORG" => $row['NRORG']
	    );
	    $stmt = $connection->prepare($sql);
	    $stmt->execute($params);
	}
	
	public function updateAutPerfilAcesso($connection, $row){
	    $sql = "UPDATE AUTPERFILACESSO SET NRPERFILACESSO = :NRPERFILACESSO WHERE IDAUTPERFACESSO = :IDAUTPERFACESSO AND NRORG = :NRORG";
	    $params = array(
	        "NRPERFILACESSO"  => $row["NRPERFILACESSO"],
	        "IDAUTPERFACESSO" => $row["IDAUTPERFACESSO"],
	        "NRORG" => $row["NRORG"]
	    );
	    $stmt = $connection->prepare($sql);
	    $stmt->execute($params);
	}
	
	public function saveGrupoOperadoresAutPerfilAcesso(DTO\Request\Row $request, DTO\Response $response){
	    try {
	        $row = $request->getRow();
	        $autperfilacesso = $this->getAutPerfilAcesso();
	        $permissions = $row['permissions'];
	        $currentRow = $row['currentRow'];
	        $orderPermissionsSelect = array_column($autperfilacesso, 'IDAUTPERFACESSO'); 
	        $basicRow = array(
	            'CDOPERINCLUSAO' => $this->environment->getCdOperador(),
	            'NRORGINCLUSAO' => $this->environment->getNrOrg(),
	            'CDOPERULTATU' => $this->environment->getCdOperador(),
	            'NRORGULTATU' => $this->environment->getNrOrg(),
	            'NRORG' => $this->environment->getNrOrg()
	        );
            $connection = $this->entityManager->getConnection();
	        foreach($permissions as $namePermission => $information){
	            $index = array_search($namePermission, $orderPermissionsSelect);
	            $connection->beginTransaction();
	            try {
    	            if($index === false && isset($currentRow[$information['value']]) && $currentRow[$information['value']]){
    	                $rowToInsert = $basicRow;
    	                $rowToInsert['NRAUTPERFACESSO'] = $this->novoCodigo->getCodeByTableName('AUTPERFILACESSO'.$rowToInsert['NRORG'], $rowToInsert['NRORG']);
    	                $rowToInsert['NRPERFILACESSO'] = $currentRow[$information['value']];
    	                $rowToInsert['IDAUTPERFACESSO'] = $namePermission;
    	                $rowToInsert['DSAUTPERFACESSO'] = $information['label'];
    	                $this->insertAutPerfilAcesso($connection, $rowToInsert);
    	            } else if(!isset($currentRow[$information['value']]) || (isset($currentRow[$information['value']]) && !$currentRow[$information['value']])){
    	                $rowToDelete = $basicRow;
    	                $rowToDelete['IDAUTPERFACESSO'] = $namePermission;
    	                $this->deleteAutPerfilAcesso($connection, $rowToDelete);
    	            } else if(isset($currentRow[$information['value']]) && $currentRow[$information['value']] !==  $autperfilacesso[$index]['NRPERFILACESSO']){
    	                $rowToUpdate = $basicRow;
    	                $rowToUpdate["NRPERFILACESSO"] = $currentRow[$information['value']];
    	                $rowToUpdate['IDAUTPERFACESSO'] = $namePermission;
    	                $this->updateAutPerfilAcesso($connection, $rowToUpdate);
    	            }
    	            $connection->commit();
	            } catch (\Exception $e){
	                $connection->rollback();
	                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
	            }
	        }
	    } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
	
	public function saveGrupoOperadores(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows();
            $grupos = array("CDGRUPOPERARRV", "CDGRUPOPERRCUS", "CDGRUPOPERRECC", "CDGRUPOPERLCMR", 
                            "CDGRUPOPERACPS", "CDGRUPOPERLRCD", "CDGRUPOPERSBTP");
                            
            $ids = array("ARRV", "RCUS", "RECC", "LCMR", "ACPS", "LRCD", "SBTP");
            
            $rowParamgeral = array( array("CDGROPERRESE" => $row[0]['CDGROPERRESE'],
		                            "CDGROPERREEX" => $row[0]['CDGROPERREEX'],
		                            "CDGROPERREPA" => $row[0]['CDGROPERREPA'],
		                            "CDPARGERAL" => $row[0]['CDPARGERAL'],
		                            "NRORG" => $row[0]['NRORG'],
		                            "__is_new" => $row[0]['__is_new']
		                            ));
		                          
            $datasetParamgeral = new DataSet('datasetParamgeral', $rowParamgeral);
            $datasetParamgeral = new DTO\Request\DataSet($datasetParamgeral, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            $this->dataSourceName = 'pla01900_grupoOperParamGeral';
            self::save($datasetParamgeral, $response);
            
            foreach ($grupos as $key => &$grupo) {
                $rowAutgrupop = array( array("CDGRUPOPER" => $row[0][$grupo],
		                            "NRORG" => $row[0]['NRORG'],
		                            "IDAUTGRUPOP" => $ids[$key],
		                            "NRAUTGRUPOP" => '000',
		                            "__is_new" => $row[0]['__is_new']
		                            ));
		                            
		        $rowAutgrupop = self::verifyGrupoOperIsNew($rowAutgrupop);
                $datasetAutgrupop = new DataSet('datasetAutgrupop', $rowAutgrupop);
                $datasetAutgrupop = new DTO\Request\DataSet($datasetAutgrupop, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                $this->dataSourceName = 'pla01900_grupoOperToSave';
                
                if($row[0][$grupo]) {
                    self::save($datasetAutgrupop, $response);
                } else {
                    self::delete($datasetAutgrupop, $response);
                }
            }
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function verifyGrupoOperIsNew($rowAutgrupop){
        $sql = "SELECT * FROM AUTGRUPOP WHERE IDAUTGRUPOP = :IDAUTGRUPOP AND NRAUTGRUPOP = '000'";
        foreach ($rowAutgrupop as $key => &$data) {
            $params = array("IDAUTGRUPOP" => $rowAutgrupop[$key]['IDAUTGRUPOP']);
		    $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            $result = $query->fetchAll();
            if(empty($result)){
                $rowAutgrupop[$key]['__is_new'] = true;
            }else{
                $rowAutgrupop[$key]['__is_new'] = false;
            }
        }
        return $rowAutgrupop;
    }
	
	public function saveCardapioDiario(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);
        
        if(!isset($resposta[1]["BLOQUEIA_EDICAO"])){
            $resposta[1]["BLOQUEIA_EDICAO"] = 'N';
        }
        
        if($resposta[1]["BLOQUEIA_EDICAO"] == 'N'){
            try {
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceToSaveCardDia);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceToSaveCardDia, $persistedRows));
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasPlanejamento();
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
        }
    }
    
    public function saveCardapioSemanal(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);
        
        if(!isset($resposta[1]["BLOQUEIA_EDICAO"])){
            $resposta[1]["BLOQUEIA_EDICAO"] = 'N';
        }
        
        if($resposta[1]["BLOQUEIA_EDICAO"] == 'N'){
            try {
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceToSaveCardSem);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceToSaveCardSem, $persistedRows));
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasPlanejamento();
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
        }
    }
    
    public function saveCardapioMensal(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);
            
        if(!isset($resposta[1]["BLOQUEIA_EDICAO"])){
            $resposta[1]["BLOQUEIA_EDICAO"] = 'N';
        }
        
        if($resposta[1]["BLOQUEIA_EDICAO"] == 'N'){
            try {
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceToSaveCardMen);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceToSaveCardMen, $persistedRows));
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasPlanejamento();
                    
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
        }
    }
    
    public function saveReqRetPrato(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);
        
        if(!isset($resposta[1]["BLOQUEIA_EDICAO"])){
            $resposta[1]["BLOQUEIA_EDICAO"] = 'N';
        }
        
        if($resposta[1]["BLOQUEIA_EDICAO"] == 'N'){
            try {
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceToSaveReqRetPrato);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceToSaveReqRetPrato, $persistedRows));
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasPlanejamento();
                    
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
        }
    }
    
    public function saveConfigPra(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);
            
        if(!isset($resposta[1]["BLOQUEIA_EDICAO"])){
            $resposta[1]["BLOQUEIA_EDICAO"] = 'N';
        }
        
        if($resposta[1]["BLOQUEIA_EDICAO"] == 'N'){
            try {
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceToSaveConfigPra);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceToSaveConfigPra, $persistedRows));
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasPlanejamento();
                    
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
        }
    }
    
    public function saveProducaoDistribuicao(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);
            
        if(!isset($resposta[1]["BLOQUEIA_EDICAO"])){
            $resposta[1]["BLOQUEIA_EDICAO"] = 'N';
        }
        
        if($resposta[1]["BLOQUEIA_EDICAO"] == 'N'){
            try {
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceToSaveProdDist);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceToSaveProdDist, $persistedRows));
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasPlanejamento();
                    
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
        }
    }
    
    public function saveCalcConsPla(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);
        
        if(!isset($resposta[0]["BLOQUEIA_EDICAO"])){
            $resposta[0]["BLOQUEIA_EDICAO"] = 'N';
        }

        if($resposta[0]["BLOQUEIA_EDICAO"] == 'N'){
            try {
                $rows = $request->getDataSet()->getRows();
                if(isset($rows[0]['selectedRows'])) {
                    foreach ($rows[0]['selectedRows'] as $key => &$data) {
                        if($data['__isSelected']){
                            $data['IDPRODNAREQPREV'] = 'S';
                        }else{
                            $data['IDPRODNAREQPREV'] = 'N';
                        }
                    }
                }
                
                $rowsProducao = $rows[0]['selectedRows'];
                $datasetProducao = new DataSet('datasetProducao', $rowsProducao);
                $datasetProducao = new DTO\Request\DataSet($datasetProducao, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                $this->dataSourceName = 'pla01900_prodSemAlteracao';
                self::save($datasetProducao, $response);

                $datasetCalcConsPla = new DataSet('datasetCalcConsPla', $rows);
                $datasetCalcConsPla = new DTO\Request\DataSet($datasetCalcConsPla, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                $this->dataSourceName = 'pla01900_calcConsPla';
                self::save($datasetCalcConsPla, $response);
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasFilial();
                    
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
        }
    }
    
    public function saveDadosTransporte(DTO\Request\DataSet $request, DTO\Response $response) {
        
        try {
            $rows = $request->getDataSet()->getRows();
            if(isset($rows[0]['changedRows'])) {
                foreach ($rows[0]['changedRows'] as $key => &$data) {
                    $rows[0]['changedRows'][$key]['CDFILIAL'] = $rows[0]['CDFILIAL'];
                }
                $rowsGrid = $rows[0]['changedRows'];
                $rowsGrid = self::verifyIsNew($rowsGrid);
                $datasetGrid = new DataSet('datasetGird', $rowsGrid);
                $datasetGrid = new DTO\Request\DataSet($datasetGrid, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                $this->dataSourceName = 'pla01900_dadosTransporteGridToSave';
                self::save($datasetGrid, $response);
            }
            
            $row = $request->getDataSet()->getRows()[0];
            $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);

            if(!isset($resposta[1]["BLOQUEIA_EDICAO"])){
                $resposta[1]["BLOQUEIA_EDICAO"] = 'N';
            }
            
            
            if($resposta[1]["BLOQUEIA_EDICAO"] == 'N'){
                $row = $rows[0]->getArrayCopy();
                $dataset = new DataSet('dataset', array($row));
                $dataset = new DTO\Request\DataSet($dataset, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                $this->dataSourceName = 'pla01900_dadosTransporte';
                self::save($dataset, $response);
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasPlanejamento();
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
        
    }
    
    public function verifyIsNew($rowsGrid){
        $sql = "SELECT * FROM UNIDMEDIFILI WHERE SGUNIDADE = :SGUNIDADE AND CDFILIAL = :CDFILIAL";
        foreach ($rowsGrid as $key => &$data) {
            $params = array("SGUNIDADE" => $rowsGrid[$key]['SGUNIDADE'],
                            "CDFILIAL" => $rowsGrid[$key]['CDFILIAL']);
		    $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            $result = $query->fetchAll();
            if(empty($result)){
                $rowsGrid[$key]['__is_new'] = true;
            }else{
                $rowsGrid[$key]['__is_new'] = false;
            }
        }
        return $rowsGrid;
    }
    
    
    public function savePlanejamento(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);
        try {
            if(!isset($resposta[0]["BLOQUEIA_EDICAO"])){
                $resposta[0]["BLOQUEIA_EDICAO"] = 'N';
            }
            
            if(!isset($resposta[1]["BLOQUEIA_EDICAO"])){
                $resposta[1]["BLOQUEIA_EDICAO"] = 'N';
            }

            if($resposta[1]["BLOQUEIA_EDICAO"] == 'N'){
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceToSavePla);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceToSavePla, $persistedRows));
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasPlanejamento();
                
            }
            
            if($resposta[0]["BLOQUEIA_EDICAO"] == 'N'){
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceToSavePlaParamFilial);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceToSavePlaParamFilial, $persistedRows));
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasFilial();
            }
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
        
        
    }
    
    
    public function saveSafraCardapio(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);
        
        if(!isset($resposta[1]["BLOQUEIA_EDICAO"])){
            $resposta[1]["BLOQUEIA_EDICAO"] = 'N';
        }
        
        if($resposta[1]["BLOQUEIA_EDICAO"] == 'N'){
            try {
                $dataSet = $request->getDataSet();
                $dataSet->getRows()[0]['__is_new'] = false;
                $dataSet->setDataSourceName($this->dataSourceToSaveSafraCard);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceToSaveSafraCard, $persistedRows));
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasPlanejamento();
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
        }
    }
    
    public function saveElaboraCardapio(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);
            
        if(!isset($resposta[1]["BLOQUEIA_EDICAO"])){
            $resposta[1]["BLOQUEIA_EDICAO"] = 'N';
        }
        
        if($resposta[1]["BLOQUEIA_EDICAO"] == 'N'){
            try {
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceToSaveElabCard);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceToSaveElabCard, $persistedRows));
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasPlanejamento();
                    
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
        }
    }
    
    public function saveServicoPla(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            foreach ($rows as $key => &$data) {
                if($data['__isSelected']){
                    $data['IDCONSSERVCA'] = 'S';
                }else{
                    $data['IDCONSSERVCA'] = 'N';
                }
            }
            $dataset = new DataSet('dataset', $rows);
            $dataset = new DTO\Request\DataSet($dataset, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            $this->dataSourceName = 'pla01900_servicoPla';
            self::save($dataset, $response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function saveServicoSafraCard(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            foreach ($rows as $key => &$data) {
                if($data['__isSelected']){
                    $data['IDCARDDESCENTR'] = 'S';
                }else{
                    $data['IDCARDDESCENTR'] = 'N';
                }
            }
            $dataset = new DataSet('dataset', $rows);
            $dataset = new DTO\Request\DataSet($dataset, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            $this->dataSourceName = 'pla01900_servicoSafraCard';
            self::save($dataset, $response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function saveServicoElabCard(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);
            
        if(!isset($resposta[1]["BLOQUEIA_EDICAO"])){
            $resposta[1]["BLOQUEIA_EDICAO"] = 'N';
        }
        
        if($resposta[1]["BLOQUEIA_EDICAO"] == 'N'){
            try {
                $rows = $request->getDataSet()->getRows();
                foreach ($rows as $key => &$data) {
                    if($data['__isSelected']){
                        $data['IDUTPRATRELAC'] = 'S';
                    }else{
                        $data['IDUTPRATRELAC'] = 'N';
                    }
                }
                $dataset = new DataSet('dataset', $rows);
                $dataset = new DTO\Request\DataSet($dataset, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                $this->dataSourceName = 'pla01900_servicoElabCard';
                self::save($dataset, $response);
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
        }
    }
    
    public function saveEfetivos(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $resposta = self::verifyFilial(new DTO\Request\Row($row, null, null, null), $response);

        if(!isset($resposta[1]["BLOQUEIA_EDICAO"])){
            $resposta[1]["BLOQUEIA_EDICAO"] = 'N';
        }
        
        if($resposta[1]["BLOQUEIA_EDICAO"] == 'N'){
            try {
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceToSaveEfetivos);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceToSaveEfetivos, $persistedRows));
                
                if($resposta[0]["FILIAL_PADRAO"] == 'S')
                    self::updateFiliaisFilhasPlanejamento();
                    
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
        }
    }
    
    public function exportParamUnidade(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $nrLista = "";
        $filialOrigem = $row['CDFILIALORIGEM'];
        if($row['ALLCDFILIAL'] != 'T') $nrLista = $this->util->insertLista($row['NMFILIAL']);
        $result = $this->Pla01900ParametroDePlanejamentoService->exportParamUnidade($request, $response, $nrLista, $filialOrigem, $row);
        if($row['ALLCDFILIAL'] != 'T') $this->util->deleteLista($nrLista);
    }
    
     public function verificaParamPlaneja(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $this->Pla01900ParametroDePlanejamentoService->verifyExistParamPlaneja($row);
    }
    
    public function insertParamGeral(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
		$sql = "INSERT INTO PARAMGERAL (CDPARGERAL) VALUES ('0001')";
		$params = array();
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        $response->addDataSet(new DataSet("INSERTPARAMPLANEJA", $result));
    }
    
    public function saveMessage(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        if($row['__is_new'] === true){
            $this->Pla01900ParametroDePlanejamentoService->insertMessage($row);
        }else{
            $this->Pla01900ParametroDePlanejamentoService->updateMessage($row);
        }
    }
}